package com.nisum.ecomproduct.controller;

import com.nisum.ecomproduct.model.Product;
import com.nisum.ecomproduct.repository.ProductRepository;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ActiveProfiles;

import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.springframework.boot.test.context.SpringBootTest.WebEnvironment.DEFINED_PORT;

@SpringBootTest(webEnvironment = DEFINED_PORT)
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@ActiveProfiles("test")
public class ProductControllerIntegrationTest {

    @LocalServerPort
    private int port;

    private String url;

    private final String randomId = "someHexString";

    @Autowired
    TestRestTemplate testRestTemplate;

    @Autowired
    ProductRepository productRepository;

    @BeforeAll
    public void setUp() {
        initializeDb();
        url = "http://localhost:" + port;
    }

    @Test
    void saveProduct_returnsProduct_201() {
        Product product = getProductRequestObject();
        ResponseEntity<Product> productResponseEntity = testRestTemplate.postForEntity("/save", product, Product.class);

        assertEquals(HttpStatus.CREATED, productResponseEntity.getStatusCode());
        Product productResponse = productResponseEntity.getBody();
        assertNotNull(productResponse);
        assertNotNull(productResponse.getId());
    }

    @Test
    void saveProducts_returnsProducts_201() {
        List<Product> products = getProductRequestObjectList();

        HttpEntity<List<Product>> productsHttpEntity = new HttpEntity<>(products);
        ResponseEntity<List<Product>> productListResponseEntity = testRestTemplate.exchange("/saveall", HttpMethod.POST, productsHttpEntity, new ParameterizedTypeReference<List<Product>>() {
        });

        assertEquals(HttpStatus.CREATED, productListResponseEntity.getStatusCode());
        List<Product> productListResponse = productListResponseEntity.getBody();
        assertNotNull(productListResponse);
        productListResponse.forEach(product -> assertNotNull(product.getId()));
    }

    private Product getProductRequestObject() {
        Product product = new Product();
        product.setName("Dell Mouse");
        product.setDescription("Input device of the computer");
        product.setPrice(300D);
        return product;
    }

    private Product getProductResponseObject() {
        Product product = getProductRequestObject();
        product.setId(randomId);
        return product;
    }

    private List<Product> getProductRequestObjectList() {
        Product product1 = new Product();
        product1.setName("Bean bag with beans - L");
        product1.setDescription("Comfortable binge");
        product1.setPrice(800D);

        Product product2 = new Product();
        product2.setName("Bean bag with beans - XL");
        product2.setDescription("Comfortable binge");
        product2.setPrice(1000D);

        Product product3 = new Product();
        product3.setName("Bean bag with beans - XXL");
        product3.setDescription("IComfortable binge");
        product3.setPrice(1600D);

        return Arrays.asList(product1, product2, product3);
    }

    private List<Product> getProductResponseObjectList() {
        List<Product> products = getProductRequestObjectList();
        products.forEach(product -> product.setId(randomId));
        return products;
    }

    private void initializeDb() {
        productRepository.deleteAll();
        productRepository.saveAll(getProductRequestObjectList());
    }

}
