package com.nisum.ecomproduct.service.impl;

import com.nisum.ecomproduct.exception.ProductNotFoundException;
import com.nisum.ecomproduct.model.Product;
import com.nisum.ecomproduct.repository.ProductRepository;
import com.nisum.ecomproduct.service.ProductService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.ActiveProfiles;

import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.when;

@SpringBootTest
@ActiveProfiles("test")
class ProductServiceImplTest {

    @Autowired
    ProductService productService;

    @MockBean
    ProductRepository productRepository;

    private final String randomId = "someHexString";

    @Test
    void saveProduct() {
        Product productToSave = getProductRequestObject();
        doReturn(getProductResponseObject()).when(productRepository).save(productToSave);
        Product savedProduct = productService.saveProduct(productToSave);
        assertNotNull(savedProduct);
        assertNotNull(savedProduct.getId());
    }

    @Test
    void saveProducts() {
        List<Product> productsToSave = getProductRequestObjectList();
        when(productRepository.saveAll(productsToSave)).thenReturn(getProductResponseObjectList());
        List<Product> savedProducts = productService.saveProducts(productsToSave);
        assertNotNull(savedProducts);
        savedProducts.forEach(savedProduct -> {
            assertNotNull(savedProduct);
            assertNotNull(savedProduct.getId());
        });
    }

    @Test()
    void deleteProductById_throwsProductNotFoundException() {
        when(productRepository.existsById(randomId)).thenReturn(false);
        assertThrows(ProductNotFoundException.class, () -> productService.deleteProductById(randomId));
    }

    private Product getProductRequestObject() {
        Product product = new Product();
        product.setName("Dell Mouse");
        product.setDescription("Input device of the computer");
        product.setPrice(300D);
        return product;
    }

    private Product getProductResponseObject() {
        Product product = getProductRequestObject();
        product.setId(randomId);
        return product;
    }

    private List<Product> getProductRequestObjectList() {
        Product product1 = new Product();
        product1.setName("Bean bag with beans - L");
        product1.setDescription("Comfortable binge");
        product1.setPrice(800D);

        Product product2 = new Product();
        product2.setName("Bean bag with beans - XL");
        product2.setDescription("Comfortable binge");
        product2.setPrice(1000D);

        Product product3 = new Product();
        product3.setName("Bean bag with beans - XXL");
        product3.setDescription("IComfortable binge");
        product3.setPrice(1600D);

        return Arrays.asList(product1, product2, product3);
    }

    private List<Product> getProductResponseObjectList() {
        List<Product> products = getProductRequestObjectList();
        products.forEach(product -> product.setId(randomId));
        return products;
    }
}